;;; event.lisp --- Event helper functions

;; ----------------------------------------
;; -- Adding and removing listeners

;;;
;; Add an event listener to the current entity. This will will execute all code
;; in the macro body and will set %SELF% to be the current entity's ID.
;;
;; This should be called within an entity's setup script.
;; 
;; @category event
;;
;; @param symbol EVENT The event identifier to listen to.
;; @param &rest  BODY  The function to execute when EVENT is fired.
(defmacro (listen event &rest body)
    `(add-event-listener ,%self% ,event '(progn ,@body)))

;;;
;; Remove an event listener from the the current entity.
;; 
;; This should only be called within an entity's script as it references %SELF%.
;; 
;; @category event
;;
;; @param symbol EVENT The event identifier to unlisten from.
(defmacro (unlisten event)
    `(remove-event-listener ,%self% ,event))

;;;
;; Add an event listener that will execute BODY when EVENT is fired. This does
;; not set %SELF% so can be used as a general listener.
;; 
;; @category event
;;
;; @param symbol EVENT The event identifier to listen to.
;; @param &rest  BODY  The function to execute when EVENT is fired.
(defmacro (event:listen event &rest body)
    `(add-event-listener nil ,event '(progn ,@body)))

;;;
;; Emit an event with NAME from SENDER with additional DATA.
;; 
;; @category event
;;
;; @param symbol NAME   The identifier of the event to send.
;; @param string SENDER Optional ID of the entity that sent the event. If NIL it 
;;                      will be treated as a global event.
;; @param mixed DATA    Optional object containing any data that should be
;;                      passed along in the event. 
(defun event:emit (name sender data)
  "Emit an event with NAME from SENDER with additional DATA."
  (emit-event name sender data))
